# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2012-06-15.      #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generator git on tinkerforge.com                   #
#############################################################

module Tinkerforge
  # Device for controlling Stacks and four Bricklets
  class BrickMaster < Device

    FUNCTION_GET_STACK_VOLTAGE = 1 # :nodoc:
    FUNCTION_GET_STACK_CURRENT = 2 # :nodoc:
    FUNCTION_SET_EXTENSION_TYPE = 3 # :nodoc:
    FUNCTION_GET_EXTENSION_TYPE = 4 # :nodoc:
    FUNCTION_IS_CHIBI_PRESENT = 5 # :nodoc:
    FUNCTION_SET_CHIBI_ADDRESS = 6 # :nodoc:
    FUNCTION_GET_CHIBI_ADDRESS = 7 # :nodoc:
    FUNCTION_SET_CHIBI_MASTER_ADDRESS = 8 # :nodoc:
    FUNCTION_GET_CHIBI_MASTER_ADDRESS = 9 # :nodoc:
    FUNCTION_SET_CHIBI_SLAVE_ADDRESS = 10 # :nodoc:
    FUNCTION_GET_CHIBI_SLAVE_ADDRESS = 11 # :nodoc:
    FUNCTION_GET_CHIBI_SIGNAL_STRENGTH = 12 # :nodoc:
    FUNCTION_GET_CHIBI_ERROR_LOG = 13 # :nodoc:
    FUNCTION_SET_CHIBI_FREQUENCY = 14 # :nodoc:
    FUNCTION_GET_CHIBI_FREQUENCY = 15 # :nodoc:
    FUNCTION_SET_CHIBI_CHANNEL = 16 # :nodoc:
    FUNCTION_GET_CHIBI_CHANNEL = 17 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt>. This object can
    # then be added to the IP connection.
    def initialize(uid)
      super uid

      @expected_name = 'Master Brick'

      @binding_version = [1, 1, 0]

    end

    # Returns the stack voltage in mV. The stack voltage is the
    # voltage that is supplied via the stack, i.e. it is given by a 
    # Step-Down or Step-Up Power Supply.
    def get_stack_voltage
      send_request(FUNCTION_GET_STACK_VOLTAGE, [], '', 2, 'S')
    end

    # Returns the stack current in mA. The stack current is the
    # current that is drawn via the stack, i.e. it is given by a
    # Step-Down or Step-Up Power Supply.
    def get_stack_current
      send_request(FUNCTION_GET_STACK_CURRENT, [], '', 2, 'S')
    end

    # Writes the extension type to the EEPROM of a specified extension. 
    # The extension is either 0 or 1 (0 is the on the bottom, 1 is the on on top, 
    # if only one extension is present use 0).
    # 
    # Possible extension types:
    # 
    #  "Type", "Description"
    # 
    #  "1",    "Chibi"
    #  "2",    "RS485"
    # 
    # The extension type is already set when bought and it can be set with the 
    # Brick Viewer, it is unlikely that you need this function.
    # 
    # The value will be saved in the EEPROM of the Chibi Extension, it does not
    # have to be set on every startup.
    def set_extension_type(extension, exttype)
      send_request(FUNCTION_SET_EXTENSION_TYPE, [extension, exttype], 'C L', 0, '')
    end

    # Returns the extension type for a given extension as set by 
    # BrickMaster#set_extension_type.
    def get_extension_type(extension)
      send_request(FUNCTION_GET_EXTENSION_TYPE, [extension], 'C', 4, 'L')
    end

    # Returns true if a Chibi Extension is available to be used by the Master.
    def is_chibi_present
      send_request(FUNCTION_IS_CHIBI_PRESENT, [], '', 1, '?')
    end

    # Sets the address (1-255) belonging to the Chibi Extension.
    # 
    # It is possible to set the address with the Brick Viewer and it will be 
    # saved in the EEPROM of the Chibi Extension, it does not
    # have to be set on every startup.
    def set_chibi_address(address)
      send_request(FUNCTION_SET_CHIBI_ADDRESS, [address], 'C', 0, '')
    end

    # Returns the address as set by BrickMaster#set_chibi_address.
    def get_chibi_address
      send_request(FUNCTION_GET_CHIBI_ADDRESS, [], '', 1, 'C')
    end

    # Sets the address (1-255) of the Chibi Master. This address is used if the
    # Chibi Extension is used as slave (i.e. it does not have a USB connection).
    # 
    # It is possible to set the address with the Brick Viewer and it will be 
    # saved in the EEPROM of the Chibi Extension, it does not
    # have to be set on every startup.
    def set_chibi_master_address(address)
      send_request(FUNCTION_SET_CHIBI_MASTER_ADDRESS, [address], 'C', 0, '')
    end

    # Returns the address as set by BrickMaster#set_chibi_master_address.
    def get_chibi_master_address
      send_request(FUNCTION_GET_CHIBI_MASTER_ADDRESS, [], '', 1, 'C')
    end

    # Sets up to 256 slave addresses. The address numeration has to be used 
    # ascending from 0. For example: If you use the Chibi Extension in Master mode
    # (i.e. the stack has an USB connection) and you want to talk to three other
    # Chibi stacks with the IDs 17, 23, and 42, you should call with "(0, 17),
    # (1, 23) and (2, 42)".
    # 
    # It is possible to set the addresses with the Brick Viewer and it will be 
    # saved in the EEPROM of the Chibi Extension, they don't
    # have to be set on every startup.
    def set_chibi_slave_address(num, address)
      send_request(FUNCTION_SET_CHIBI_SLAVE_ADDRESS, [num, address], 'C C', 0, '')
    end

    # Returns the slave address for a given num as set by 
    # BrickMaster#set_chibi_slave_address.
    def get_chibi_slave_address(num)
      send_request(FUNCTION_GET_CHIBI_SLAVE_ADDRESS, [num], 'C', 1, 'C')
    end

    # Returns the signal strength in dBm. The signal strength updates every time a
    # packet is received.
    def get_chibi_signal_strength
      send_request(FUNCTION_GET_CHIBI_SIGNAL_STRENGTH, [], '', 1, 'C')
    end

    # Returns underrun, CRC error, no ACK and overflow error counts of the Chibi
    # communication. If these errors start rising, it is likely that either the
    # distance between two Chibi stacks is becoming too big or there are
    # interferences.
    def get_chibi_error_log
      send_request(FUNCTION_GET_CHIBI_ERROR_LOG, [], '', 8, 'S S S S')
    end

    # Sets the Chibi frequency range for the Chibi Extension. Possible values are:
    # 
    #  "Type", "Description"
    # 
    #  "0",    "OQPSK 868Mhz (Europe)"
    #  "1",    "OQPSK 915Mhz (US)"
    #  "2",    "OQPSK 780Mhz (China)"
    #  "3",    "BPSK40 915Mhz"
    # 
    # It is possible to set the frequency with the Brick Viewer and it will be 
    # saved in the EEPROM of the Chibi Extension, it does not
    # have to be set on every startup.
    def set_chibi_frequency(frequency)
      send_request(FUNCTION_SET_CHIBI_FREQUENCY, [frequency], 'C', 0, '')
    end

    # Returns the frequency value as set by BrickMaster#set_chibi_frequency.
    def get_chibi_frequency
      send_request(FUNCTION_GET_CHIBI_FREQUENCY, [], '', 1, 'C')
    end

    # Sets the channel used by the Chibi Extension. Possible channels are
    # different for different frequencies:
    # 
    #  "Frequency",             "Possible Channels"
    # 
    #  "OQPSK 868Mhz (Europe)", "0"
    #  "OQPSK 915Mhz (US)",     "1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
    #  "OQPSK 780Mhz (China)",  "0, 1, 2, 3"
    #  "BPSK40 915Mhz",         "1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
    # 
    # It is possible to set the frequency with the Brick Viewer and it will be 
    # saved in the EEPROM of the Chibi Extension, it does not
    # have to be set on every startup.
    def set_chibi_channel(channel)
      send_request(FUNCTION_SET_CHIBI_CHANNEL, [channel], 'C', 0, '')
    end

    # Returns the channel as set by BrickMaster#set_chibi_channel.
    def get_chibi_channel
      send_request(FUNCTION_GET_CHIBI_CHANNEL, [], '', 1, 'C')
    end

  end
end
