# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2012-06-15.      #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generator git on tinkerforge.com                   #
#############################################################

module Tinkerforge
  # Device for controlling up to 4 general purpose input/output pins
  class BrickletIO4 < Device

    # This callback is triggered whenever a change of the voltage level is detected
    # on pins where the interrupt was activated with :func:`SetPortInterrupt`.
    # 
    # The values are a bit mask that specifies which interrupts occurred
    # and the current value bit mask of the port.
    # 
    # For example:
    # 
    #  * (1, 1) means that an interrupt on pin 0 occurred and
    #    currently pin 0 is high and pins 1-3 are low.
    #  * (9, 14) means that an interrupt on pins 0 and 3
    #    occurred and currently pin 0 is low and pins 1-3 are high.
    CALLBACK_INTERRUPT = 9

    FUNCTION_SET_VALUE = 1 # :nodoc:
    FUNCTION_GET_VALUE = 2 # :nodoc:
    FUNCTION_SET_CONFIGURATION = 3 # :nodoc:
    FUNCTION_GET_CONFIGURATION = 4 # :nodoc:
    FUNCTION_SET_DEBOUNCE_PERIOD = 5 # :nodoc:
    FUNCTION_GET_DEBOUNCE_PERIOD = 6 # :nodoc:
    FUNCTION_SET_INTERRUPT = 7 # :nodoc:
    FUNCTION_GET_INTERRUPT = 8 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt>. This object can
    # then be added to the IP connection.
    def initialize(uid)
      super uid

      @expected_name = 'IO-4 Bricklet'

      @binding_version = [1, 0, 0]

      @callback_formats[CALLBACK_INTERRUPT] = 'C C'
    end

    # Sets the output value (high or low) with a bit mask. The bit mask
    # is 4 bit long, "true" refers to high and "false" refers to low.
    # 
    # For example: The value 0b0011 will turn the pins 0-1 high and the
    # pins 2-3 low.
    # 
    #  .. note::
    #   This function does nothing for pins that are configured as input.
    #   Pull up resistors can be switched on with :func:`SetPortConfiguration`.
    def set_value(value_mask)
      send_request(FUNCTION_SET_VALUE, [value_mask], 'C', 0, '')
    end

    # Returns a bit mask of the values that are currently measured.
    # This function works if the pin is configured to input
    # as well as if it is configured to output.
    def get_value
      send_request(FUNCTION_GET_VALUE, [], '', 1, 'C')
    end

    # Configures the value and direction of the specified pins. Possible directions
    # are "i" and "o" for input and output.
    # 
    # If the direction is configured as output, the value is either high or low
    # (set as true or false).
    # 
    # If the direction is configured as input, the value is either pull up or
    # default (set as true or false).
    # 
    # For example: 
    # 
    #  * (15, 'i', true) will set all pins of as input pull up. 
    #  * (8, 'i', false) will set pin 3 of as input default (floating if nothing is connected). 
    #  * (3, 'o', false) will set pins 0 and 1 as output low.
    #  * (4, 'o', true) will set pin 2 of as output high.
    def set_configuration(pin_mask, direction, value)
      send_request(FUNCTION_SET_CONFIGURATION, [pin_mask, direction, value], 'C k ?', 0, '')
    end

    # Returns a value bit mask and a direction bit mask.
    # 
    # For example: A return value of 0b0011 and 0b0101 for
    # direction and value means that:
    # 
    #  * pin 0 is configured as input pull up, 
    #  * pin 1 is configured as input default,
    #  * pin 2 is configured as output high
    #  * and pin 3 is are configured as output low.
    def get_configuration
      send_request(FUNCTION_GET_CONFIGURATION, [], '', 2, 'C C')
    end

    # Sets the debounce period of the CALLBACK_INTERRUPT callback in ms.
    # 
    # For example: If you set this value to 100, you will get the interrupt
    # maximal every 100ms. This is necessary if something that bounces is
    # connected to the IO-4 Bricklet, such as a button.
    # 
    # The default value is 100.
    def set_debounce_period(debounce)
      send_request(FUNCTION_SET_DEBOUNCE_PERIOD, [debounce], 'L', 0, '')
    end

    # Returns the debounce period as set by BrickletIO4#set_debounce_period.
    def get_debounce_period
      send_request(FUNCTION_GET_DEBOUNCE_PERIOD, [], '', 4, 'L')
    end

    # Sets the pins on which an interrupt is activated with a bit mask. 
    # Interrupts are triggered on changes of the voltage level of the pin,
    # i.e. changes from high to low and low to high.
    # 
    # For example: An interrupt bit mask of 9 will enable the interrupt for 
    # pins 0 and 3.
    # 
    # The interrupt is delivered with the callback CALLBACK_INTERRUPT.
    def set_interrupt(interrupt_mask)
      send_request(FUNCTION_SET_INTERRUPT, [interrupt_mask], 'C', 0, '')
    end

    # Returns the interrupt bit mask as set by :func:`SetPortInterrupt`.
    def get_interrupt
      send_request(FUNCTION_GET_INTERRUPT, [], '', 1, 'C')
    end

    # Registers a callback with ID <tt>cb</tt> to the block <tt>block</tt>.
    def register_callback(cb, &block)
      callback = block
      @registered_callbacks[cb] = callback
    end
  end
end
